package xyz.lychee.lagfixer.libs.adventure.key;

import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;
import xyz.lychee.lagfixer.libs.examination.string.StringExaminer;

final class KeyedValueImpl<T> implements Examinable, KeyedValue<T> {
   private final Key key;
   private final T value;

   KeyedValueImpl(final Key key, final T value) {
      this.key = key;
      this.value = value;
   }

   public @NotNull Key key() {
      return this.key;
   }

   public @NotNull T value() {
      return this.value;
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (other != null && this.getClass() == other.getClass()) {
         KeyedValueImpl<?> that = (KeyedValueImpl)other;
         return this.key.equals(that.key) && this.value.equals(that.value);
      } else {
         return false;
      }
   }

   public int hashCode() {
      int result = this.key.hashCode();
      result = 31 * result + this.value.hashCode();
      return result;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("key", this.key), ExaminableProperty.of("value", this.value));
   }

   public String toString() {
      return (String)this.examine(StringExaminer.simpleEscaping());
   }
}
